{*******************************************************}
{                                                       }
{                      Tachyon Unit                     }
{    Vector Raster Geographic Information Synthesis     }
{                     VOICE  ..  Tracer                 }
{                     GRIP ICE .. Tongs                 }
{                Digital Terrain Mapping                }
{               Image Locatable Holographics            }
{                          SOS MAP                      }
{  Surreal Object Synthesis Multimedia Analysis Product }
{                   Fractal3D  Life MOW                 }
{       Copyright (c) 1995,2006  Ivan Lee Herring       }
{                                                       }
{*******************************************************}
unit buzAlcon;
{NAME                      ALASKA CONFORMAL
PURPOSE:	Transforms input longitude and latitude to Easting and Northing
  for the Alaska Conformal projection.  The longitude and latitude
  must be in radians.  The Easting and Northing values will
  be returned in meters.
PROGRAMMER              DATE
----------              ----
T. Mittan		March, 1993
This function was adapted from the Alaska Conformal projection code
(FORTRAN) in the General Cartographic Transformation Package software
which is available from the U.S. Geological Survey National Mapping Division.
ALGORITHM REFERENCES
1.  "New Equal-Area Map Projections for Noncircular Regions", John P. Snyder,
    The American Cartographer, Vol 15, No. 4, October 1988, pp. 341-355.
2.  Snyder, John P., "Map Projections--A Working Manual", U.S. Geological
    Survey Professional Paper 1395 (Supersedes USGS Bulletin 1532), United
    State Government Printing Office, Washington D.C., 1987.
3.  "Software Documentation for GCTP General Cartographic Transformation
    Package", U.S. Geological Survey National Mapping Division, May 1982.
#include "gctpc/cproj.h"
#include "gctpc/report.h"}

interface

uses Math;

//* Variables common to all subroutines in this code file
var
  r_major, //* major axis
    r_minor, //* minor axis
    lon_center, //* Center longitude (projection center)
    lat_center, //* center latitude
    false_easting, //* x offset in meters
    false_northing, //* y offset in meters
    sin_p26,
    cos_p26,
    e: double;
  acoef: array[0..7] of double;
  bcoef: array[0..7] of double;
  n: Integer;

function
  alconforint(r_maj, r_min, false_east, false_north: double):
    Integer;

function
  alconfor(lon, lat: double; var x, y: double): Integer;

function
  alconinvint(r_maj, r_min, false_east, false_north: double):
    Integer;

function
  alconinv(x, y: double; var lon, lat: double): Integer;

implementation

uses buzGlobal;

function
  alconforint(r_maj, r_min, false_east, false_north: double):
    Integer;
{Initialize the ALASKA CONFORMAL projection
long alconforint(
double r_maj,/* Major axis
double r_min,/* Minor axis
double false_east,/* x offset in meters
double false_north)/* y offset in meters}
var
{ i:Integer;
 temp,}
  es,
    chi,
    esphi: double;
begin
//* Place parameters in static storage for common use
  r_major := r_maj;
  r_minor := r_min;
  false_easting := false_east;
  false_northing := false_north;
  lon_center := -152.0 * D2R;
  lat_center := 64.0 * D2R;
  n := 6;

  es := 0.006768657997291094;
  e := sqrt(es);

  acoef[1] := 0.9945303;
  acoef[2] := 0.0052083;
  acoef[3] := 0.0072721;
  acoef[4] := -0.0151089;
  acoef[5] := 0.0642675;
  acoef[6] := 0.3582802;
  bcoef[1] := 0.0;
  bcoef[2] := -0.0027404;
  bcoef[3] := 0.0048181;
  bcoef[4] := -0.1932526;
  bcoef[5] := -0.1381226;
  bcoef[6] := -0.2884586;

  esphi := e * sin(lat_center);
  chi := 2.0 * arctan(tan((HALF_PI + lat_center) / 2.0) *
    power(((1.0 - esphi) / (1.0 + esphi)), (e / 2.0))) - HALF_PI;
  sin_p26 := Sin(chi);
  cos_p26 := Cos(chi);
  {sincos(chi, sin_p26, cos_p26);}


//* Report parameters to the user  -----------------------------*/
  ptitle('ALASKA CONFORMAL');
  radius2(r_major, r_minor);
  cenlon(lon_center);
  cenlat(lat_center);
  offsetp(false_easting, false_northing);
  result := (OK);
end;


function
  alconfor(lon, lat: double; var x, y: double): Integer;
{ ALASKA CONFORMAL forward equations--mapping lat,long to x,y
long alconfor(
double lon,/* (I) Longitude
double lat,/* (I) Latitude
double *x,/* (O) X projection coordinate
double *y)/* (O) Y projection coordinate}
var
{double adjust_lon(double x);}
  dlon, sinlon, coslon, sinphi, cosphi,
    esphi, g, s, xp, yp, ar, ai, br, bi, {e,} arn, ain, chi, r:
      double;
  j: Integer;
begin
//* Forward equations
  dlon := AdjustLon(lon - lon_center);
//* caluclate x' and y' for Oblique Stereographic Proj for LAT/LONG
  sinlon := Sin(dlon);
  coslon := Cos(dlon);
  {sincos(dlon, sinlon, coslon);}
  esphi := e * sin(lat);
  chi := 2.0 * arctan(tan((HALF_PI + lat) / 2.0) *
    power(((1.0 - esphi) / (1.0 + esphi)), (e / 2.0))) - HALF_PI;
  sinphi := Sin(chi);
  cosphi := Cos(chi);
  {sincos(chi, sinphi, cosphi); }
  g := sin_p26 * sinphi + cos_p26 * cosphi * coslon;
  s := 2.0 / (1.0 + g);
  xp := s * cosphi * sinlon;
  yp := s * (cos_p26 * sinphi - sin_p26 * cosphi * coslon);

//* Use Knuth algorithm for summing complex terms, to convert
//   Oblique Stereographic to Modified-Stereographic coord
  r := xp + xp;
  s := xp * xp + yp * yp;
  ar := acoef[n];
  ai := bcoef[n];
  br := acoef[n - 1];
  bi := bcoef[n - 1];
{for j :=2 to <= n do}
  j := 2;
  arn := br + r * ar;
  ain := bi + r * ai;
  while j < n do
  begin
    inc(j);
    arn := br + r * ar;
    ain := bi + r * ai;
    if (j < n) then
    begin
      br := acoef[n - j] - s * ar;
      bi := bcoef[n - j] - s * ai;
      ar := arn;
      ai := ain;
    end;
  end;
  br := -s * ar;
  bi := -s * ai;
  ar := arn;
  ai := ain;
  x := (xp * ar - yp * ai + br) * r_major + false_easting;
  y := (yp * ar + xp * ai + bi) * r_major + false_northing;
  result := (OK);
end;



function
  alconinvint(r_maj, r_min, false_east, false_north: double):
    Integer;
{ Initialize the ALASKA CONFORMAL projection
long alconinvint(
double r_maj,/* Major axis
double r_min,/* Minor axis
double false_east,/* x offset in meters
double false_north)/* y offset in meters}
var
{ i:Integer;}
{ temp,       }
  es,
    chi,
    esphi: double;
begin
//* Place parameters in static storage for common use
  r_major := r_maj;
  r_minor := r_min;
  false_easting := false_east;
  false_northing := false_north;
  lon_center := -152.0 * D2R;
  lat_center := 64.0 * D2R;
  n := 6;

{temp := r_minor / r_major; }
  es := 0.006768657997291094;
  e := sqrt(es);

  acoef[1] := 0.9945303;
  acoef[2] := 0.0052083;
  acoef[3] := 0.0072721;
  acoef[4] := -0.0151089;
  acoef[5] := 0.0642675;
  acoef[6] := 0.3582802;
  bcoef[1] := 0.0;
  bcoef[2] := -0.0027404;
  bcoef[3] := 0.0048181;
  bcoef[4] := -0.1932526;
  bcoef[5] := -0.1381226;
  bcoef[6] := -0.2884586;
  esphi := e * sin(lat_center);
  chi := 2.0 * arctan(tan((HALF_PI + lat_center) / 2.0) *
    power(((1.0 - esphi) / (1.0 + esphi)), (e / 2.0))) - HALF_PI;
  sin_p26 := Sin(chi);
  cos_p26 := Cos(chi);
  {sincos(chi, sin_p26, cos_p26);}

//* Report parameters to the user
  ptitle('ALASKA CONFORMAL');
  radius2(r_major, r_minor);
  cenlon(lon_center);
  cenlat(lat_center);
  offsetp(false_easting, false_northing);
  result := (OK);
end;


function
  alconinv(x, y: double; var lon, lat: double): Integer;
{ ALASKA CONFORMAL inverse equations--mapping x,y to lat/long
long alconinv(
double x,/* (O) X projection coordinate
double y,/* (O) Y projection coordinate
double *lon,/* (I) Longitude
double *lat)/* (I) Latitude}
var
{double adjust_lon(double x);
double asinz(double con);}
  dlon, sinlon, coslon, esphi, r, s, br, bi, ai,
    ar, ci, cr, di, dr, arn, ain, crn, cin,
    fxyr, fxyi, fpxyr, fpxyi, xp, yp, den, dxp, dyp, ds,
    z, cosz, sinz, rh, chi, dphi, phi: double;
  j, nn: Integer;
begin
//* Inverse equations
  x := (x - false_easting) / r_major;
  y := (y - false_northing) / r_major;
  xp := x;
  yp := y;
  nn := 0;

//* Use Knuth algorithm for summing complex terms, to convert Modified-
//   Stereographic conformal to Oblique Stereographic coordinates.

  while (ds > EPSLN) do
  begin
    r := xp + xp;
    s := xp * xp + yp * yp;
    ar := acoef[n];
    ai := bcoef[n];
    br := acoef[n - 1];
    bi := bcoef[n - 1];
    cr := {(double)}(n) * ar;
    ci := {(double)}(n) * ai;
    dr := {(double)}(n - 1) * br;
    di := {(double)}(n - 1) * bi;

{  for (j = 2; j <= n; j++)}
    arn := br + r * ar;
    ain := bi + r * ai;
    j := 2;
    while (j <= n) do
    begin
      inc(j);
      arn := br + r * ar;
      ain := bi + r * ai;
      if (j < n) then
      begin
        br := acoef[n - j] - s * ar;
        bi := bcoef[n - j] - s * ai;
        ar := arn;
        ai := ain;
        crn := dr + r * cr;
        cin := di + r * ci;
        dr := {(double)}(n - j) * acoef[n - j] - s * cr;
        di := {(double)}(n - j) * bcoef[n - j] - s * ci;
        cr := crn;
        ci := cin;
      end;
    end;
    br := -s * ar;
    bi := -s * ai;
    ar := arn;
    ai := ain;
    fxyr := xp * ar - yp * ai + br - x;
    fxyi := yp * ar + xp * ai + bi - y;
    fpxyr := xp * cr - yp * ci + dr;
    fpxyi := yp * cr + xp * ci + ci;
    den := fpxyr * fpxyr + fpxyi * fpxyi;
    dxp := -(fxyr * fpxyr + fxyi * fpxyi) / den;
    dyp := -(fxyi * fpxyr - fxyr * fpxyi) / den;
    xp := xp + dxp;
    yp := yp + dyp;
    ds := abs(dxp) + abs(dyp);
    inc(nn) {++};
    if (nn > 20) then
    begin
      {PError('Too many iterations in inverse', 'alcon-inv');}
      DoProjError(235,235,0,0,0);
      result := (235);
    end;
  end;
{while (ds > EPSLN);}

//* convert Oblique Stereographic coordinates to LAT/LONG
  rh := sqrt(xp * xp + yp * yp);
  z := 2.0 * arctan(rh / 2.0);
  sinz := Sin(z);
  cosz := Cos(z);
  {sincos(z, sinz, cosz);}
  lon := lon_center;
  if (abs(rh) <= EPSLN) then
  begin
    lat := lat_center;
    result := (OK);
    Exit;
  end;
  chi := asinz(cosz * sin_p26 + (yp * sinz * cos_p26) / rh);
  nn := 0;
  phi := chi;

  repeat
    begin
      esphi := e * sin(phi);
      dphi := 2.0 * arctan(tan((HALF_PI + chi) / 2.0) *
        power(((1.0 + esphi) / (1.0 - esphi)), (e / 2.0))) - HALF_PI
          - phi;
      phi := phi + dphi;
      inc(nn) {++};
      if (nn > 20) then
      begin
        {PError('Too many iterations in inverse"', 'alcon-inv');}
        DoProjError(235,235,0,0,0);
        result := (236);
        Exit;
      end;
    end;
  until (not (abs(dphi) > EPSLN));
{while(abs(dphi) > EPSLN);}

  lat := phi;
  lon := AdjustLon(lon_center
    + arctan((xp * sinz) /
    (rh * cos_p26 * cosz - yp * sin_p26 * sinz)));

  result := (OK);
end;

end.
