{*******************************************************}
{                                                       }
{                      Tachyon Unit                     }
{    Vector Raster Geographic Information Synthesis     }
{                     VOICE  ..  Tracer                 }
{                     GRIP ICE .. Tongs                 }
{                Digital Terrain Mapping                }
{               Image Locatable Holographics            }
{                          SOS MAP                      }
{  Surreal Object Synthesis Multimedia Analysis Product }
{                   Fractal3D  Life MOW                 }
{       Copyright (c) 1995,2006  Ivan Lee Herring       }
{                                                       }
{*******************************************************}
unit buzGood;
{*****************************************************************
NAME                        GOODE'S HOMOLOSINE
PURPOSE:	Transforms input longitude and latitude to Easting and
  Northing for the Goode's Homolosine projection.  The
  longitude and latitude must be in radians.  The Easting
  and Northing values will be returned in meters.
PROGRAMMER              DATE
----------              ----
D. Steinwand, EROS      May, 1991; Updated Sept, 1992; Updated Feb 1993
S. Nelson, EDC		Jun, 1993	Make changes in precision and number
     of iterations.
ALGORITHM REFERENCES
1.  Snyder, John P. and Voxland, Philip M., "An Album of Map Projections",
    U.S. Geological Survey Professional Paper 1453 , United State Government
    Printing Office, Washington D.C., 1989.
2.  Snyder, John P., "Map Projections--A Working Manual", U.S. Geological
    Survey Professional Paper 1395 (Supersedes USGS Bulletin 1532), United
    State Government Printing Office, Washington D.C., 1987.
3.  Goode, J.P., 1925,  The Homolosine projection:  a new device for
    portraying the Earth's surface entire:  Assoc. Am. Geographers, Annals,
    v. 15, p. 119-125
*******************************************************************************/
#include "gctpc/cproj.h"
#include "gctpc/report.h"}

interface
uses Math;

//* Variables common to all subroutines in this code file
var
  R: double; //* Radius of the earth (sphere) */
  //* Central meridians, one for each region */
  lon_center: array[0..12] of double;
  //* False easting, one for each region */
  feast: array[0..12] of double;



function
  goodforint(r: double): Integer;

function
  goodfor(lon, lat: double; var x, y: double): Integer;

function
  goodinvint(r: double): Integer;

function
  goodinv(x, y: double; var lon, lat: double): Integer;

implementation

uses buzGlobal;

function
  goodforint(r: double): Integer;
{ Initialize the Goode`s Homolosine projection
long goodforint(double r)}
// (I) Radius of the earth (sphere) */
begin
//* Place parameters in static storage for common use
  R := r;

//* Initialize central meridians for each of the 12 regions
  lon_center[0] := -1.74532925199; //* -100.0 degrees */
  lon_center[1] := -1.74532925199; //* -100.0 degrees */
  lon_center[2] := 0.523598775598; //*   30.0 degrees */
  lon_center[3] := 0.523598775598; //*   30.0 degrees */
  lon_center[4] := -2.79252680319; //* -160.0 degrees */
  lon_center[5] := -1.0471975512; //*  -60.0 degrees */
  lon_center[6] := -2.79252680319; //* -160.0 degrees */
  lon_center[7] := -1.0471975512; //*  -60.0 degrees */
  lon_center[8] := 0.349065850399; //*   20.0 degrees */
  lon_center[9] := 2.44346095279; //*  140.0 degrees */
  lon_center[10] := 0.349065850399; //*   20.0 degrees */
  lon_center[11] := 2.44346095279; //*  140.0 degrees */

//* Initialize false eastings for each of the 12 regions
  feast[0] := R * -1.74532925199;
  feast[1] := R * -1.74532925199;
  feast[2] := R * 0.523598775598;
  feast[3] := R * 0.523598775598;
  feast[4] := R * -2.79252680319;
  feast[5] := R * -1.0471975512;
  feast[6] := R * -2.79252680319;
  feast[7] := R * -1.0471975512;
  feast[8] := R * 0.349065850399;
  feast[9] := R * 2.44346095279;
  feast[10] := R * 0.349065850399;
  feast[11] := R * 2.44346095279;

//* Report parameters to the user
  ptitle('GOODE S HOMOLOSINE EQUAL-AREA');
  radius(r);
  result := (OK);
end;


function
  goodfor(lon, lat: double; var x, y: double): Integer;
{Goode`s Homolosine forward equations--mapping lat,long to x,y
  -------------------------------------------------------------*/
long
goodfor(double lon, double lat, double *x, double *y)
              /* (I) Longitude */
              /* (I) Latitude */
             /* (O) X projection coordinate */
             /* (O) Y projection coordinate */}
{double adjust_lon(double x);
/* Function to adjust longitude to -180 - 180 */}
var
  delta_lon, //* Delta longitude (Given longitude - center
    theta,
    delta_theta,
    constant: double;
  i, region: Integer;
begin
//* Forward equations  -----------------*/
  if (lat >= 0.710987989993) then //* if on or above 40 44' 11.8" */
  begin
    if (lon <= -0.698131700798) then
      region := 0 //* If to the left of -40 */
    else region := 2;
  end
  else if (lat >= 0.0) then //* Between 0.0 and 40 44' 11.8" */
  begin
    if (lon <= -0.698131700798) then
      region := 1 //* If to the left of -40 */
    else region := 3;
  end
  else if (lat >= -0.710987989993) then
    //* Between 0.0 & -40 44' 11.8" */
  begin
    if (lon <= -1.74532925199) then
      region := 4 //* If between -180 and -100 */
    else if (lon <= -0.349065850399) then
      region := 5 //* If between -100 and -20 */
    else if (lon <= 1.3962634016) then
      region := 8 //* If between -20 and 80 */
    else region := 9; //* If between 80 and 180 */
  end
  else //* Below -40 44' */
  begin
    if (lon <= -1.74532925199) then
      region := 6 //* If between -180 and -100 */
    else if (lon <= -0.349065850399) then
      region := 7 //* If between -100 and -20 */
    else if (lon <= 1.3962634016) then
      region := 10 //* If between -20 and 80 */
    else region := 11; //* If between 80 and 180 */
  end;

  if ((region = 1) or (region = 3) or (region = 4) or (region = 5) or
    (region = 8) or (region = 9)) then
  begin
    delta_lon := AdjustLon(lon - lon_center[region]);
    x := feast[region] + R * delta_lon * cos(lat);
    y := R * lat;
  end
  else
  begin
    delta_lon := AdjustLon(lon - lon_center[region]);
    theta := lat;
    constant := PI * sin(lat);

//* Iterate using the Newton-Raphson method to find theta
    for i := 0 to 51 do
    begin
      delta_theta := -(theta + sin(theta) - constant) / (1.0 +
        cos(theta));
      theta := theta + delta_theta;
      if (abs(delta_theta) < EPSLN) then break;
      if (i >= 50) then
      begin
        {PError('Iteration failed to converge', 'goode-forward');}
        DoProjError(251,251,0,0,0);
        result := (251);
        exit;
      end;
    end;
    theta := theta / 2.0;

   //* If the latitude is 90 deg, force the x coordinate to be
    //  "0 + false easting" this is done here because of precision problems
   //   with "cos(theta)"
   //   ------------------------------------------------------------------*/
    if (PI / 2 - abs(lat) < EPSLN) then delta_lon := 0;
    x := feast[region] + 0.900316316158 * R * delta_lon * cos(theta);
    y := R * (1.4142135623731 * sin(theta) - 0.0528035274542 *
      sign(lat));
  end;
  result := (OK);
end;


function
  goodinvint(r: double): Integer;
{ Initialize the Goode`s Homolosine projection
long goodinvint(double r)
/* (I) Radius of the earth (sphere) */}
begin
//* Place parameters in static storage for common use
  R := r;

//Initialize central meridians for each of the 12 regions
//  -------------------------------------------------------
  lon_center[0] := -1.74532925199; //* -100.0 degrees */
  lon_center[1] := -1.74532925199; //* -100.0 degrees */
  lon_center[2] := 0.523598775598; //*   30.0 degrees */
  lon_center[3] := 0.523598775598; //*   30.0 degrees */
  lon_center[4] := -2.79252680319; //* -160.0 degrees */
  lon_center[5] := -1.0471975512; //*  -60.0 degrees */
  lon_center[6] := -2.79252680319; //* -160.0 degrees */
  lon_center[7] := -1.0471975512; //*  -60.0 degrees */
  lon_center[8] := 0.349065850399; //*   20.0 degrees */
  lon_center[9] := 2.44346095279; //*  140.0 degrees */
  lon_center[10] := 0.349065850399; //*   20.0 degrees */
  lon_center[11] := 2.44346095279; //*  140.0 degrees */

//* Initialize false eastings for each of the 12 regions
//  ----------------------------------------------------*/
  feast[0] := R * -1.74532925199;
  feast[1] := R * -1.74532925199;
  feast[2] := R * 0.523598775598;
  feast[3] := R * 0.523598775598;
  feast[4] := R * -2.79252680319;
  feast[5] := R * -1.0471975512;
  feast[6] := R * -2.79252680319;
  feast[7] := R * -1.0471975512;
  feast[8] := R * 0.349065850399;
  feast[9] := R * 2.44346095279;
  feast[10] := R * 0.349065850399;
  feast[11] := R * 2.44346095279;

//* Report parameters to the user  -----------------------------*/
  ptitle('GOODE S HOMOLOSINE EQUAL-AREA');
  radius(r);
  result := (OK);
end;

function
  goodinv(x, y: double; var lon, lat: double): Integer;
{Goode`s Homolosine inverse equations--mapping x,y to lat,long
long goodinv(double x, double y, double *lon, double *lat)
           /* (I) X projection coordinate */
           /* (I) Y projection coordinate */
              /* (O) Longitude */
              /* (O) Latitude */}
{double adjust_lon(double x);
//* Function to adjust longitude to -180 - 180 */}
var
  arg,
    theta,
    temp: double;
  region: Integer;
begin
  result := (OK);
// Inverse equations  -----------------*/
  if (y >= R * 0.710987989993) then //* if on or above 40 44' 11.8" */
  begin
    if (x <= R * -0.698131700798) then
      region := 0 //* If to the left of -40 */
    else region := 2;
  end
  else if (y >= 0.0) then //* Between 0.0 and 40 44' 11.8" */
  begin
    if (x <= R * -0.698131700798) then
      region := 1 //* If to the left of -40 */
    else region := 3;
  end
  else if (y >= R * -0.710987989993) then
    //* Between 0.0 & -40 44' 11.8" */
  begin
    if (x <= R * -1.74532925199) then
      region := 4 //* If between -180 and -100 */
    else if (x <= R * -0.349065850399) then
      region := 5 //* If between -100 and -20 */
    else if (x <= R * 1.3962634016) then
      region := 8 //* If between -20 and 80 */
    else region := 9; //* If between 80 and 180 */
  end
  else //* Below -40 44' 11.8" */
  begin
    if (x <= R * -1.74532925199) then
      region := 6 //* If between -180 and -100 */
    else if (x <= R * -0.349065850399) then
      region := 7 //* If between -100 and -20 */
    else if (x <= R * 1.3962634016) then
      region := 10 //* If between -20 and 80 */
    else region := 11; //* If between 80 and 180 */
  end;
  x := x - feast[region];

  if ((region = 1) or (region = 3) or (region = 4)
    or (region = 5) or (region = 8) or (region = 9)) then
  begin
    lat := y / R;
    if (abs(lat) > HALF_PI) then
    begin
      {PError('Input data error', 'goode-inverse'); }
      DoProjError(252,252,0,0,0);
      result := (252);
      exit;
    end;
    temp := abs(lat) - HALF_PI;
    if (abs(temp) > EPSLN) then
    begin
      temp := lon_center[region] + x / (R * cos(lat));
      lon := AdjustLon(temp);
    end
    else lon := lon_center[region];
  end
  else
  begin
    arg := (y + 0.0528035274542 * R * sign(y)) / (1.4142135623731 *
      R);
    if (abs(arg) > 1.0) then result := (IN_BREAK);
    theta := arcsin(arg);
    lon := lon_center[region] + (x / (0.900316316158 * R *
      cos(theta)));
    if (lon < -(PI + EPSLN)) then result := (IN_BREAK);
    arg := (2.0 * theta + sin(2.0 * theta)) / PI;
    if (abs(arg) > 1.0) then result := (IN_BREAK);
    lat := arcsin(arg);
  end;
//* because of precision problems,
//long values of 180 deg and -180 deg may be mixed.
  if (((x < 0) and (PI - lon < EPSLN))
    or ((x > 0) and (PI + lon < EPSLN))) then
    lon := -(lon);

//* Are we in a interrupted area?

//If so, result :=  status code of IN_BREAK.
//  ---------------------------------------------------------------------*/
  if ((region = 0) and ((lon < -(PI + EPSLN)) or (lon >
    -0.698131700798))) then
    result := (IN_BREAK);
  if ((region = 1) and ((lon < -(PI + EPSLN)) or (lon >
    -0.698131700798))) then
    result := (IN_BREAK);
  if ((region = 2) and ((lon < -0.698131700798) or (lon > PI +
    EPSLN))) then
    result := (IN_BREAK);
  if ((region = 3) and ((lon < -0.698131700798) or (lon > PI +
    EPSLN))) then
    result := (IN_BREAK);
  if ((region = 4) and ((lon < -(PI + EPSLN)) or (lon >
    -1.74532925199))) then
    result := (IN_BREAK);
  if ((region = 5) and ((lon < -1.74532925199) or (lon >
    -0.349065850399))) then
    result := (IN_BREAK);
  if ((region = 6) and ((lon < -(PI + EPSLN)) or (lon >
    -1.74532925199))) then
    result := (IN_BREAK);
  if ((region = 7) and ((lon < -1.74532925199) or (lon >
    -0.349065850399))) then
    result := (IN_BREAK);
  if ((region = 8) and ((lon < -0.349065850399) or (lon >
    1.3962634016))) then
    result := (IN_BREAK);
  if ((region = 9) and ((lon < 1.3962634016) or (lon > PI + EPSLN)))
    then
    result := (IN_BREAK);
  if ((region = 10) and ((lon < -0.349065850399) or (lon >
    1.3962634016))) then
    result := (IN_BREAK);
  if ((region = 11) and ((lon < 1.3962634016) or (lon > PI + EPSLN)))
    then
    result := (IN_BREAK);


end;



end.
