{*******************************************************}
{                                                       }
{                      Tachyon Unit                     }
{    Vector Raster Geographic Information Synthesis     }
{                     VOICE  ..  Tracer                 }
{                     GRIP ICE .. Tongs                 }
{                Digital Terrain Mapping                }
{               Image Locatable Holographics            }
{                          SOS MAP                      }
{  Surreal Object Synthesis Multimedia Analysis Product }
{                   Fractal3D  Life MOW                 }
{       Copyright (c) 1995,2006  Ivan Lee Herring       }
{                                                       }
{*******************************************************}
unit buzHam;

{*********************************************************************
NAME                            HAMMER

PURPOSE:	Transforms input longitude and latitude to Easting and
  Northing for the Hammer projection.  The longitude
  and latitude must be in radians.  The Easting and
  Northing values will be returned in meters.

PROGRAMMER              DATE
----------              ----
T. Mittan		March, 1993

This function was adapted from the Lambert Azimuthal Equal Area projection
code (FORTRAN) in the General Cartographic Transformation Package software
which is available from the U.S. Geological Survey National Mapping Division.

ALGORITHM REFERENCES

1.  "New Equal-Area Map Projections for Noncircular Regions", John P. Snyder,
    The American Cartographer, Vol 15, No. 4, October 1988, pp. 341-355.

2.  Snyder, John P., "Map Projections--A Working Manual", U.S. Geological
    Survey Professional Paper 1395 (Supersedes USGS Bulletin 1532), United
    State Government Printing Office, Washington D.C., 1987.

3.  "Software Documentation for GCTP General Cartographic Transformation
    Package", U.S. Geological Survey National Mapping Division, May 1982.
*******************************************************************************/
#include "gctpc/cproj.h"
#include "gctpc/report.h"}
interface
uses Math;
//* Variables common to all subroutines in this code file
var
  lon_center, //* Center longitude (projection center)
    RadiusES, //* Radius of the earth (sphere)
    false_easting, //* x offset in meters
    false_northing: Double; //* y offset in meters}



function
  hamforint(r, center_long, false_east, false_north: double):
    Integer;

function
  hamfor(lon, lat: double; var x, y: double): Integer;

function
  haminvint(r, center_long, false_east, false_north: double):
    Integer;

function
  haminv(x, y: double; var lon, lat: double): Integer;

implementation
uses buzGlobal;

function
  hamforint(r, center_long, false_east, false_north: double):
    Integer;
{ Initialize the HAMMER projection
  -------------------------------*/
long hamforint(double r, double center_long, double false_east, double false_north)
             /* (I) Radius of the earth (sphere)
                     /* (I) Center longitude
                    /* x offset in meters
                     /* y offset in meters}
{var i :Integer;}
begin
//* Place parameters in static storage for common use
  RadiusES := r;
  lon_center := center_long;
  false_easting := false_east;
  false_northing := false_north;

//* Report parameters to the user  -----------------------------*/
  ptitle('HAMMER');
  radius(r);
  cenlon(center_long);
  offsetp(false_easting, false_northing);
  result := (OK);
end;

function
  hamfor(lon, lat: double; var x, y: double): Integer;
{HAMMER forward equations--mapping lat,long to x,y
long hamfor(double lon, double lat, double *x, double *y)
              /* (I) Longitude */
              /* (I) Latitude */
             /* (O) X projection coordinate */
             /* (O) Y projection coordinate */}
var dlon, fac: Double;
begin
{double dlon;
double adjust_lon(double x);
double fac;}
//* Forward equations  -----------------*/
  dlon := AdjustLon(lon - lon_center);

  fac := RadiusES * 1.414213562 / sqrt(1.0 + cos(lat) * cos(dlon /
    2.0));
  x := false_easting + fac * 2.0 * cos(lat) * sin(dlon / 2.0);
  y := false_northing + fac * sin(lat);

  result := (OK);
end;



function
  haminvint(r, center_long, false_east, false_north: double):
    Integer;
{Initialize the HAMMER projection
long haminvint(double r, double center_long, double false_east, double false_north)

             /* (I) Radius of the earth (sphere) 	*/
                     /* (I) Center longitude 		*/
                    /* x offset in meters			*/
                     /* y offset in meters			*/}
{var i : integer;}
begin
//* Place parameters in static storage for common use
  RadiusES := r;
  lon_center := center_long;
  false_easting := false_east;
  false_northing := false_north;

//* Report parameters to the user  -----------------------------*/
  ptitle('HAMMER');
  radius(r);
  cenlon(center_long);
  offsetp(false_easting, false_northing);
  result := (OK);
end;

function
  haminv(x, y: double; var lon, lat: double): Integer;
{HAMMER inverse equations--mapping x,y to lat/long
long haminv(double x, double y, double *lon, double *lat)
            /* (O) X projection coordinate */
            /* (O) Y projection coordinate */
               /* (I) Longitude */
               /* (I) Latitude */}
{double dlon;
double adjust_lon(double x);
double asinz(double con);}
var {dlon,}  fac: Double;
begin
//* Inverse equations  -----------------*/
  x := x - false_easting;
  y := y - false_northing;
  fac := sqrt(4.0 * RadiusES * RadiusES - (x * x) / 4.0 - y * y) /
    2.0;
  lon := AdjustLon(lon_center + 2.0 *
    arctan((x * fac) / (2.0 * RadiusES * RadiusES - x * x / 4 - y *
      y)));
  lat := asinz(y * fac / RadiusES / RadiusES);
  result := (OK);
end;



end.
