{*******************************************************}
{                                                       }
{                      Tachyon Unit                     }
{    Vector Raster Geographic Information Synthesis     }
{                     VOICE  ..  Tracer                 }
{                     GRIP ICE .. Tongs                 }
{                Digital Terrain Mapping                }
{               Image Locatable Holographics            }
{                          SOS MAP                      }
{  Surreal Object Synthesis Multimedia Analysis Product }
{                   Fractal3D  Life MOW                 }
{       Copyright (c) 1995,2006  Ivan Lee Herring       }
{                                                       }
{*******************************************************}
unit buzMer;
{************************************************************************
NAME                            MERCATOR

PURPOSE:	Transforms input longitude and latitude to Easting and
  Northing for the Mercator projection.  The
  longitude and latitude must be in radians.  The Easting
  and Northing values will be returned in meters.

PROGRAMMER              DATE
----------              ----
D. Steinwand, EROS      Nov, 1991
T. Mittan		Mar, 1993

ALGORITHM REFERENCES

1.  Snyder, John P., "Map Projections--A Working Manual", U.S. Geological
    Survey Professional Paper 1395 (Supersedes USGS Bulletin 1532), United
    State Government Printing Office, Washington D.C., 1987.

2.  Snyder, John P. and Voxland, Philip M., "An Album of Map Projections",
    U.S. Geological Survey Professional Paper 1453 , United State Government
    Printing Office, Washington D.C., 1989.
*******************************************************************************/
#include "gctpc/cproj.h"
#include "gctpc/report.h"}
interface
uses Math;
//* Variables common to all subroutines in this code file
var
  r_major, //* major axis
    r_minor, //* minor axis
    lon_center, //* Center longitude (projection center)
    lat_origin, //* center latitude
    e, es, //* eccentricity constants
    m1, //* small value m
    false_northing, //* y offset in meters
    false_easting: Double; //* x offset in meters


function
  merforint(r_maj, r_min, center_lon, center_lat,
  false_east, false_north: double): Integer;

function
  merfor(lon, lat: double; var x, y: double): Integer;

function
  merinvint(r_maj, r_min, center_lon, center_lat,
  false_east, false_north: double): Integer;

function
  merinv(x, y: double; var lon, lat: double): Integer;

implementation
uses buzGlobal;

function
  merforint(r_maj, r_min, center_lon, center_lat,
  false_east, false_north: double): Integer;
{Initialize the Mercator projection
long merforint(double r_maj, double r_min,
double center_lon, double center_lat,
double false_east, double false_north)
                /* major axis			*/
                /* minor axis			*/
                    /* center longitude		*/
                    /* center latitude		*/
                    /* x offset in meters		*/
                     /* y offset in meters		*/}
var
  temp: double; //* temporary variable		*/
{double e0fn(double x),e1fn(double x),e2fn(double x),e3fn(double x);
  /* 	functions	*/}
begin
//* Place parameters in static storage for common use

  r_major := r_maj;
  r_minor := r_min;
  lon_center := center_lon;
  lat_origin := center_lat;
  false_northing := false_north;
  false_easting := false_east;

  temp := r_minor / r_major;
  es := 1.0 - SQR(temp);
  e := sqrt(es);
  m1 := cos(center_lat) / (sqrt(1.0 - es * sin(center_lat) *
    sin(center_lat)));

//* Report parameters to the user

  ptitle('MERCATOR');
  radius2(r_major, r_minor);
  cenlonmer(lon_center);
  origin(lat_origin);
  offsetp(false_easting, false_northing);
  result := (OK);
end;

function
  merfor(lon, lat: double; var x, y: double): Integer;
{Mercator forward equations--mapping lat,long to x,y
long merfor(double lon, double lat, double *x, double *y)
              /* (I) Longitude 		*/
              /* (I) Latitude 		*/
             /* (O) X projection coordinate 	*/
             /* (O) Y projection coordinate 	*/}
var
{double adjust_lon(double x);
/* Function to adjust longitude to -180 - 180 	*/
double tsfnz(double eccent, double phi, double sinphi);
/* function to compute t			*/}
  ts, //* small t value
    sinphi: double; //* sin value
begin
//* Forward equations
  If ( bIgnoreErrorsCB and ((abs(abs(lat) - HALF_PI) <= EPSLN)))  then
    begin
     If lat < 0 then lat:= lat+  ((abs(abs(lat) - HALF_PI)-EPSLN))
     else lat:= lat-  ((abs(abs(lat) - HALF_PI)-EPSLN));
    end;
 {Did it work?}
  if (abs(abs(lat) - HALF_PI) <= EPSLN) then
  begin
    {PError('Transformation cannot be computed at the poles',
      'mer-forward');}
    DoProjError(53,53,0,0,0);
    result := (53);
    exit;
  end
  else
  begin
    sinphi := sin(lat);
    ts := tsfnz(e, lat, sinphi);
    x := false_easting + r_major * m1 * AdjustLon(lon - lon_center);
    y := false_northing - r_major * m1 * ln(ts);
  end;
  result := (OK);
end;



function
  merinvint(r_maj, r_min, center_lon, center_lat,
  false_east, false_north: double): Integer;
{Initialize the Mercator projection
long merinvint(double r_maj, double r_min,
double center_lon, double center_lat,
double false_east, double false_north)
                /* major axis
                /* minor axis
                    /* center longitude
                    /* center latitude
                    /* x offset in meters
                     /* y offset in meters}
var
  temp: double; //* temporary variable		*/
{double e0fn(double x),e1fn(double x),e2fn(double x),e3fn(double x);
 /* 	functions	*/}
begin
//* Place parameters in static storage for common use

  r_major := r_maj;
  r_minor := r_min;
  lon_center := center_lon;
  lat_origin := center_lat;
  false_northing := false_north;
  false_easting := false_east;

  temp := r_minor / r_major;
  es := 1.0 - SQR(temp);
  e := sqrt(es);
  m1 := cos(center_lat) / (sqrt(1.0 - es * sin(center_lat) *
    sin(center_lat)));

//* Report parameters to the user

  ptitle('MERCATOR');
  radius2(r_major, r_minor);
  cenlonmer(lon_center);
  origin(lat_origin);
  offsetp(false_easting, false_northing);
  result := (OK);
end;



function
  merinv(x, y: double; var lon, lat: double): Integer;
{Mercator inverse equations--mapping x,y to lat/long
long
merinv(double x, double y, double *lon, double *lat)
            /* (O) X projection coordinate 	*/
            /* (O) Y projection coordinate 	*/
               /* (I) Longitude 		*/
               /* (I) Latitude 		*/}
var
{double adjust_lon(double x);
/* Function to adjust longitude to -180 - 180 	*/
double tsfnz(double eccent, double phi, double sinphi);
/* function to compute t
double phi2z(double eccent, double ts, long int *flag);
 /* function to compute phi}
  ts //* small t value				*/
{ sin_phii}: Double; //* sin value					*/
  flag: Integer; //* error flag 					*/
begin
//* Inverse equations
  flag := 0;
  x := x - false_easting;
  y := y - false_northing;
  ts := exp(-y / (r_major * m1));
  lat := phi2z(e, ts, flag);
  if (flag <> 0) then begin result := (flag); exit; end;
  lon := AdjustLon(lon_center + x / (r_major * m1));

  result := (OK);
end;



end.
