{*******************************************************}
{                                                       }
{                      Tachyon Unit                     }
{    Vector Raster Geographic Information Synthesis     }
{                     VOICE  ..  Tracer                 }
{                     GRIP ICE .. Tongs                 }
{                Digital Terrain Mapping                }
{               Image Locatable Holographics            }
{                          SOS MAP                      }
{  Surreal Object Synthesis Multimedia Analysis Product }
{                   Fractal3D  Life MOW                 }
{       Copyright (c) 1995,2006  Ivan Lee Herring       }
{                                                       }
{*******************************************************}
unit buzPoly;

{*****************************************************************
NAME                             POLYCONIC
PURPOSE:	Transforms input longitude and latitude to Easting and
  Northing for the Polyconic projection.  The
  longitude and latitude must be in radians.  The Easting
  and Northing values will be returned in meters.
PROGRAMMER              DATE
----------              ----
T. Mittan		Mar, 1993
ALGORITHM REFERENCES
1.  Snyder, John P., "Map Projections--A Working Manual", U.S. Geological
    Survey Professional Paper 1395 (Supersedes USGS Bulletin 1532), United
    State Government Printing Office, Washington D.C., 1987.
2.  Snyder, John P. and Voxland, Philip M., "An Album of Map Projections",
    U.S. Geological Survey Professional Paper 1453 , United State Government
    Printing Office, Washington D.C., 1989.

#include "gctpc/cproj.h"
#include "gctpc/report.h"}
interface
uses Math;
//* Variables common to all subroutines in this code file
var
  r_major, //* major axis 				*/
    r_minor, //* minor axis 				*/
    lon_center, //* Center longitude (projection center) */
    lat_origin, //* center latitude			*/
    e0, e1, e2, e3, //* eccentricity constants		*/
    e, es, esp, //* eccentricity constants		*/
    ml0, //* small value m			*/
    false_northing, //* y offset in meters			*/
    false_easting: Double; //* x offset in meters			*/


function
  polyforint(r_maj, r_min, center_lon, center_lat,
  false_east, false_north: double): Integer;

function
  polyfor(lon, lat: double; var x, y: double): Integer;

function
  polyinvint(r_maj, r_min, center_lon, center_lat,
  false_east, false_north: double): Integer;

function
  polyinv(x, y: double; var lon, lat: double): Integer;

implementation
uses buzGlobal;


function
  polyforint(r_maj, r_min, center_lon, center_lat,
  false_east, false_north: double): Integer;
{Initialize the POLYCONIC projection
  ----------------------------------*/
long
polyforint(double r_maj, double r_min,
double center_lon, double center_lat,
double false_east, double false_north)
                /* major axis
                /* minor axis
                    /* center longitude
                    /* center latitude
                    /* x offset in meters
                     /* y offset in meters}
var temp: double; //* temporary variable		*/
{double e0fn(double x),e1fn(double x),
e2fn(double x),e3fn(double x),
mlfn(double e0, double e1, double e2, double e3, double phi);
 /* functions	*/ }
begin
//* Place parameters in static storage for common use
  r_major := r_maj;
  r_minor := r_min;
  lon_center := center_lon;
  lat_origin := center_lat;
  false_northing := false_north;
  false_easting := false_east;

  temp := r_minor / r_major;
  es := 1.0 - SQR(temp);
  e := sqrt(es);
  e0 := e0fn(es);
  e1 := e1fn(es);
  e2 := e2fn(es);
  e3 := e3fn(es);
  ml0 := mlfn(e0, e1, e2, e3, lat_origin);

//* Report parameters to the user
  ptitle('POLYCONIC');
  radius2(r_major, r_minor);
  cenlonmer(lon_center);
  origin(lat_origin);
  offsetp(false_easting, false_northing);
  result := (OK);
end;

function
  polyfor(lon, lat: double; var x, y: double): Integer;
{Polyconic forward equations--mapping lat,long to x,y
long
polyfor(double lon, double lat, double *x, double *y)
              /* (I) Longitude 		*/
              /* (I) Latitude 		*/
             /* (O) X projection coordinate 	*/
             /* (O) Y projection coordinate 	*/}
var
{double adjust_lon(double x);
 /* Function to adjust longitude to -180 - 180 	*/
double mlfn(double e0, double e1, double e2, double e3, double phi);		/* function to compute small m			*/
double msfnz(double eccent, double sinphi, double cosphi);
  /* function to compute m			*/}
  sinphi, cosphi, //* sin and cos value
 { al,  }//* temporary values
 { c,}//* temporary values
  con, ml, //* cone constant, small m
    ms: double; //* small m
begin
//* Forward equations
  con := AdjustLon(lon - lon_center);
  if (abs(lat) <= 0.0000001) then
  begin
    x := false_easting + r_major * con;
    y := false_northing - r_major * ml0;
  end
  else
  begin
    sinphi := Sin(lat);
    cosphi := Cos(lat);
    {sincos(lat, sinphi, cosphi);}
    ml := mlfn(e0, e1, e2, e3, lat);
    ms := msfnz(e, sinphi, cosphi);
    con := con * sinphi;
    x := false_easting + r_major * ms * sin(con) / sinphi;
    y := false_northing + r_major * (ml - ml0 + ms * (1.0 - cos(con))
      / sinphi);
  end;
  result := (OK);
end;


function
  polyinvint(r_maj, r_min, center_lon, center_lat,
  false_east, false_north: double): Integer;
{Initialize the POLYCONIC projection
  ----------------------------------*/
long
polyinvint(double r_maj, double r_min,
double center_lon, double center_lat,
double false_east, double false_north)
                /* major axis			*/
                /* minor axis			*/
                    /* center longitude		*/
                    /* center latitude		*/
                    /* x offset in meters		*/
                     /* y offset in meters		*/}
var
  temp: double; //* temporary variable		*/
{double e0fn(double x),e1fn(double x),
e2fn(double x),e3fn(double x),
mlfn(double e0, double e1, double e2, double e3, double phi);
 /* functions	*/}
begin
//* Place parameters in static storage for common use

  r_major := r_maj;
  r_minor := r_min;
  lon_center := center_lon;
  lat_origin := center_lat;
  false_northing := false_north;
  false_easting := false_east;

  temp := r_minor / r_major;
  es := 1.0 - SQR(temp);
  e := sqrt(es);
  e0 := e0fn(es);
  e1 := e1fn(es);
  e2 := e2fn(es);
  e3 := e3fn(es);
  ml0 := mlfn(e0, e1, e2, e3, lat_origin);

//* Report parameters to the user

  ptitle('POLYCONIC');
  radius2(r_major, r_minor);
  cenlonmer(lon_center);
  origin(lat_origin);
  offsetp(false_easting, false_northing);
  result := (OK);
end;

function
  polyinv(x, y: double; var lon, lat: double): Integer;
{ Polyconic inverse equations--mapping x,y to lat/long
long
polyinv(double x, double y, double *lon, double *lat)
            /* (O) X projection coordinate
            /* (O) Y projection coordinate
               /* (I) Longitude
               /* (I) Latitude}
var
{double adjust_lon(double x);
 /* Function to adjust longitude to -180 - 180 	*/
double asinz(double con);
double mlfn(double e0, double e1, double e2, double e3, double phi);
 /* function to compute small m			*/
double phi4z(double eccent, double e0, double e1, double e2,
 double e3, double a, double b, double *c, double *phi);
 /* function to compute latitude			*/}
{ sin_phi, cos_phi,}//* sin and cos value
  al, //* temporary values
    b, //* temporary values
    c: double; //* temporary values
{ con, ml:double;}//* cone constant, small m
  iflg: Integer; //* error flag
begin
//* Inverse equations

  x := x - false_easting;
  y := y - false_northing;
  al := ml0 + y / r_major;
{iflg := 0;}
  result := (OK);
  if (abs(al) <= 0.0000001) then
  begin
    lon := x / r_major + lon_center;
    lat := 0.0;
  end
  else
  begin
    b := al * al + (x / r_major) * (x / r_major);
    iflg := round(phi4z(es, e0, e1, e2, e3, al, b, c, lat));
    if (iflg <> 0) then result := (74);
    lon := AdjustLon((asinz(x * c / r_major) / sin(lat)) +
      lon_center);
  end
end;


end.
