{*******************************************************}
{                                                       }
{                      Tachyon Unit                     }
{    Vector Raster Geographic Information Synthesis     }
{                     VOICE  ..  Tracer                 }
{                     GRIP ICE .. Tongs                 }
{                Digital Terrain Mapping                }
{               Image Locatable Holographics            }
{                          SOS MAP                      }
{  Surreal Object Synthesis Multimedia Analysis Product }
{                   Fractal3D  Life MOW                 }
{       Copyright (c) 1995,2006  Ivan Lee Herring       }
{                                                       }
{*******************************************************}
unit buzVandg;

{*******************************************************************
NAME                    VAN DER GRINTEN

PURPOSE:	Transforms input longitude and latitude to Easting and
  Northing for the Van der Grinten projection.  The
  longitude and latitude must be in radians.  The Easting
  and Northing values will be returned in meters.

PROGRAMMER              DATE
----------              ----
T. Mittan		March, 1993

This function was adapted from the Lambert Azimuthal Equal Area projection
code (FORTRAN) in the General Cartographic Transformation Package software
which is available from the U.S. Geological Survey National Mapping Division.

ALGORITHM REFERENCES

1.  "New Equal-Area Map Projections for Noncircular Regions", John P. Snyder,
    The American Cartographer, Vol 15, No. 4, October 1988, pp. 341-355.

2.  Snyder, John P., "Map Projections--A Working Manual", U.S. Geological
    Survey Professional Paper 1395 (Supersedes USGS Bulletin 1532), United
    State Government Printing Office, Washington D.C., 1987.

3.  "Software Documentation for GCTP General Cartographic Transformation
    Package", U.S. Geological Survey National Mapping Division, May 1982.
*******************************************************************************/
#include "gctpc/cproj.h"
#include "gctpc/report.h"}
interface

uses Math;
//* Variables common to all subroutines in this code file
var
  lon_center, //* Center longitude (projection center) */
    R, //* Radius of the earth (sphere)	 	*/
    false_easting, //* x offset in meters			*/
    false_northing: Double; //* y offset in meters			*/

function
  vandgforint(r, center_long, false_east, false_north: double):
    Integer;

function
  vandgfor(lon, lat: double; var x, y: double): Integer;

function
  vandginvint(r, center_long, false_east, false_north: double):
    Integer;

function
  vandginv(x, y: double; var lon, lat: double): Integer;

implementation
uses buzGlobal;


function
  vandgforint(r, center_long, false_east, false_north: double):
    Integer;
{ Initialize the Van Der Grinten projection
long vandgforint(double r, double center_long,
double false_east, double false_north)
             /* (I) Radius of the earth (sphere) 	*/
                     /* (I) Center longitude 		*/
                    /* x offset in meters			*/
                     /* y offset in meters			*/}
begin
//* Place parameters in static storage for common use
  R := r;
  lon_center := center_long;
  false_easting := false_east;
  false_northing := false_north;

//* Report parameters to the user
  ptitle('VAN DER GRINTEN');
  radius(r);
  cenlon(center_long);
  offsetp(false_easting, false_northing);
  result := (OK);
end;


function
  vandgfor(lon, lat: double; var x, y: double): Integer;
{Van Der Grinten forward equations--mapping lat,long to x,y
long vandgfor(double lon, double lat, double *x, double *y)
              /* (I) Longitude */
              /* (I) Latitude */
             /* (O) X projection coordinate */
             /* (O) Y projection coordinate */}

var
  dlon,
{double AdjustLon(double x);
double asinz(double con);}
  theta,
    al, asq,
    g, gsq,
    m, msq,
    con,
    costh, sinth {, temp}: double;
begin
//* Forward equations

  dlon := AdjustLon(lon - lon_center);

  if (abs(lat) <= EPSLN) then
  begin
    x := false_easting + R * dlon;
    y := false_northing;
    result := (OK);
    exit;
  end;
  theta := asinz(2.0 * abs(lat / PI));
  if ((abs(dlon) <= EPSLN) or (abs(abs(lat) - HALF_PI) <= EPSLN))
    then
  begin
    x := false_easting;
    if (lat >= 0) then
      y := false_northing + PI * R * tan(0.5 * theta)
    else
      y := false_northing + PI * R * -tan(0.5 * theta);
    result := (OK);
    exit;
  end;
  al := 0.5 * abs((PI / dlon) - (dlon / PI));
  asq := al * al;
    sinth := Sin(theta);
    costh := Cos(theta);
  {sincos(theta, sinth, costh);}
  g := costh / (sinth + costh - 1.0);
  gsq := g * g;
  m := g * (2.0 / sinth - 1.0);
  msq := m * m;
  con := PI * R * (al * (g - msq) + sqrt(asq * (g - msq) * (g - msq)
    - (msq + asq)
    * (gsq - msq))) / (msq + asq);
  if (dlon < 0) then
    con := -con;
  x := false_easting + con;
  con := abs(con / (PI * R));
  if (lat >= 0) then
    y := false_northing + PI * R * sqrt(1.0 - con * con - 2.0 * al *
      con)
  else
    y := false_northing - PI * R * sqrt(1.0 - con * con - 2.0 * al *
      con);

  result := (OK);
end;


function
  vandginvint(r, center_long, false_east, false_north: double):
    Integer;
{Initialize the Van Der Grinten projection
long
vandginvint(double r, double center_long,
double false_east, double false_north)
             /* (I) Radius of the earth (sphere) 	*/
                     /* (I) Center longitude 		*/
                    /* x offset in meters			*/
                     /* y offset in meters}
begin
//* Place parameters in static storage for common use
  R := r;
  lon_center := center_long;
  false_easting := false_east;
  false_northing := false_north;

//* Report parameters to the user

  ptitle('VAN DER GRINTEN');
  radius(r);
  cenlon(center_long);
  offsetp(false_easting, false_northing);
  result := (OK);
end;


function
  vandginv(x, y: double; var lon, lat: double): Integer;
{ Van Der Grinten inverse equations--mapping x,y to lat/long
long
vandginv(double x, double y, double *lon, double *lat)
            /* (O) X projection coordinate */
            /* (O) Y projection coordinate */
               /* (I) Longitude */
               /* (I) Latitude */}

var
{double AdjustLon(double x);}
{ dlon,}
  xx, yy, xys, c1, c2, c3,
{ al,asq, }
  a1,
    m1,
    con,
    th1,
    d: Double;
begin
//* inverse equations

  x := x - false_easting;
  y := y - false_northing;
  con := PI * R;
  xx := x / con;
  yy := y / con;
  xys := xx * xx + yy * yy;
  c1 := -abs(yy) * (1.0 + xys);
  c2 := c1 - 2.0 * yy * yy + xx * xx;
  c3 := -2.0 * c1 + 1.0 + 2.0 * yy * yy + xys * xys;
  d := yy * yy / c3 + (2.0 * c2 * c2 * c2 / c3 / c3 / c3 - 9.0 * c1 *
    c2 / c3 / c3)
    / 27.0;
  a1 := (c1 - c2 * c2 / 3.0 / c3) / c3;
  m1 := 2.0 * sqrt(-a1 / 3.0);
  con := ((3.0 * d) / a1) / m1;
  if (abs(con) > 1.0) then
  begin
    if (con >= 0.0) then con := 1.0
    else con := -1.0;
  end;
  th1 := arccos(con) / 3.0;
  if (y >= 0) then
    lat := (-m1 * cos(th1 + PI / 3.0) - c2 / 3.0 / c3) * PI
  else
    lat := -(-m1 * cos(th1 + PI / 3.0) - c2 / 3.0 / c3) * PI;

  if (abs(xx) < EPSLN) then
  begin
    lon := lon_center;
    result := (OK);
    exit;
  end;
  lon := AdjustLon(lon_center + PI * (xys - 1.0 + sqrt(1.0 + 2.0 *
    (xx * xx - yy * yy) + xys * xys)) / 2.0 / xx);

  result := (OK);
end;



end.
